# copy_out.py
#
# Copyright (C) 2013 to 2014 Tim Marston <tim@edm.am>
#
# This file is part of stdhome (hereafter referred to as "this program").
# See http://ed.am/dev/stdhome for more information.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import filecmp, os, shutil
from walker import Walker
import stdhome.the as the


class CopyOutWalker( Walker ):
	"""The copy-out walker traverses the repo, copying its content to the home
	directory.  It is run *after* checking for conflicts that might occur during
	copy-out, so any conflicts that exist can be assumed to be unimportant and
	disposable.  In other words, this will happily overwrite stuff in the home
	directory, without regard to its content.

	Walker source:       repo
	Walker destination:  home dir
	Walker traversing:   repo (or a provided file list, which may include stuff
							that no longer exists in the repo
	"""

	def __init__( self, updated_files = None ):
		self.src_dir = the.repo.full_dir
		self.dst_dir = the.full_home_dir
		self.walk_list = updated_files if updated_files is not None else \
						 self.generate_walk_list( the.repo.full_dir )

		self.accept_symlinks = list()


	def process( self, rel_file, src, dst ):

		# directory (in repo)
		if src.type == 'd':

			# if entity doesn't exist in home dir, create directory (no need to
			# recurse, since we're copying the whole directory)
			if dst.type == '_':
				if the.verbose > 1: print "  _<d " + rel_file
				os.mkdir( dst.file )
				shutil.copystat( src.file, dst.file )
				return False

			# if entity is a directory in home dir, copy permissions, as
			# required (and recurse)
			elif dst.type == 'd':
				# TODO: should check permission and only do as necessary
				if the.verbose > 1: print "  d<d " + rel_file
				shutil.copystat( src.file, dst.file )
				return True

			# TODO: if entity is a symlink to a directory, and this is
			# acceptable, just recurse
#			elif dst.link_type == 'd' and self.accept_list.match( rel_file ):
#				if the.verbose > 1: print "  d@=d " + rel_file
#				return True

			# if entity is a file or symlink in home dir, replace it with
			# directory (no need to recurse, since we're copying the whole
			# directory)
			elif dst.type == 'f' or dst.type == 'l':
				if the.verbose > 1: print "  %s<d %s" % ( dst.type, rel_file )
				os.unlink( dst.file )
				os.mkdir( dst.file )
				shutil.copystat( src.file, dst.file )
				return False

			else:
				raise NotImplementedError()

		# file (in repo)
		if src.type == 'f':

			# if entity doesn't exist in home dir, copy file
			if dst.type == '_':
				if the.verbose > 1: print "  _<f " + rel_file
				shutil.copy( src.file, dst.file )
				shutil.copystat( src.file, dst.file )

			# if entity is a file in home dir, replace it only if it differs
			elif dst.type == 'f':
				if not filecmp.cmp( src.file, dst.file ):
					if the.verbose > 1: print "  f<f " + rel_file
					os.unlink( dst.file )
					shutil.copy( src.file, dst.file )
					shutil.copystat( src.file, dst.file )
				else:
					if the.verbose > 1: print "  f=f " + rel_file

			# if entity is a directory in home dir, replace it with file
			elif dst.type == 'd':
				if the.verbose > 1: print "  d<f " + rel_file
				shutil.rmtree( dst.file )
				shutil.copy( src.file, dst.file )
				shutil.copystat( src.file, dst.file )

			# if entity is a symlink in home dir, replace it with file
			elif dst.type == 'l':
				if the.verbose > 1: print "  l<f " + rel_file
				os.unlink( dst.file )
				shutil.copy( src.file, dst.file )
				shutil.copystat( src.file, dst.file )

			else:
				raise NotImplementedError()

		# link (in repo)
		if src.type == 'l':

			# if entity doesn't exist in home dir, copy symlink
			if dst.type == '_':
				if the.verbose > 1: print "  _<l " + rel_file
				os.symlink( os.readlink( src.file ), dst.file )

			# if entity is a symlink in home dir, replace it only if it differs
			elif dst.type == 'l':
				if os.readlink( src.file ) != os.readlink( dst.file ):
					if the.verbose > 1: print "  l<l " + rel_file
					os.unlink( dst.file )
					os.symlink( os.readlink( src.file ), dst.file )
				else:
					if the.verbose > 1: print "  l=l " + rel_file

			# if entity is a file in home dir, replace it with file
			elif dst.type == 'f':
				if the.verbose > 1: print "  f<l " + rel_file
				os.unlink( dst.file )
				os.symlink( os.readlink( src.file ), dst.file )

			# if entity is a directory in home dir, replace it with file
			elif dst.type == 'd':
				if the.verbose > 1: print "  d<l " + rel_file
				shutil.rmtree( dst.file )
				os.symlink( os.readlink( src.file ), dst.file )

			else:
				raise NotImplementedError()

		# deleted file (in repo)
		if src.type == '_':

			# if entity doesn't exist in home dir, we're good
			if dst.type == '_':
				pass;

			# if entity is a file or symlink in home dir, delete it
			if dst.type == 'f' or dst.type == 'l':
				if the.verbose > 1: print "  %s<_ %s" % ( dst.type, rel_file )
				os.unlink( dst.file )

			# if entity is a directory in home dir, delete it
			elif dst.type == 'd':
				if the.verbose > 1: print "  d<_ " + rel_file
				shutil.rmtree( dst.file )

			else:
				raise NotImplementedError()

		# non-directories can not be recursed in to
		return False
